PartialEq

Trait PartialEq 

1.6.0 (const: unstable) ยท Source
pub trait PartialEq<Rhs: PointeeSized = Self>: PointeeSized {
    // Required method
    fn eq(&self, other: &Rhs) -> bool;

    // Provided method
    fn ne(&self, other: &Rhs) -> bool { ... }
}
Trait for comparisons using the equality operator.

Implementing this trait for types provides the == and != operators for those types.

x.eq(y) can also be written x == y, and x.ne(y) can be written x != y. We use the easier-to-read infix notation in the remainder of this documentation.

This trait allows for comparisons using the equality operator, for types that do not have a full equivalence relation. For example, in floating point numbers NaN != NaN, so floating point types implement PartialEq but not Eq. Formally speaking, when Rhs == Self, this trait corresponds to a partial equivalence relation.

Implementations must ensure that eq and ne are consistent with each other:

  • a != b if and only if !(a == b).

The default implementation of ne provides this consistency and is almost always sufficient. It should not be overridden without very good reason.

If PartialOrd or Ord are also implemented for Self and Rhs, their methods must also be consistent with PartialEq (see the documentation of those traits for the exact requirements). Itโ€™s easy to accidentally make them disagree by deriving some of the traits and manually implementing others.

The equality relation == must satisfy the following conditions (for all a, b, c of type A, B, C):

  • Symmetry: if A: PartialEq<B> and B: PartialEq<A>, then a == b implies b == a; and

  • Transitivity: if A: PartialEq<B> and B: PartialEq<C> and A: PartialEq<C>, then a == b and b == c implies a == c. This must also work for longer chains, such as when A: PartialEq<B>, B: PartialEq<C>, C: PartialEq<D>, and A: PartialEq<D> all exist.

Note that the B: PartialEq<A> (symmetric) and A: PartialEq<C> (transitive) impls are not forced to exist, but these requirements apply whenever they do exist.

Violating these requirements is a logic error. The behavior resulting from a logic error is not specified, but users of the trait must ensure that such logic errors do not result in undefined behavior. This means that unsafe code must not rely on the correctness of these methods.

Cross-crate considerations

Upholding the requirements stated above can become tricky when one crate implements PartialEq for a type of another crate (i.e., to allow comparing one of its own types with a type from the standard library). The recommendation is to never implement this trait for a foreign type. In other words, such a crate should do impl PartialEq<ForeignType> for LocalType, but it should not do impl PartialEq<LocalType> for ForeignType.

This avoids the problem of transitive chains that criss-cross crate boundaries: for all local types T, you may assume that no other crate will add impls that allow comparing T == U. In other words, if other crates add impls that allow building longer transitive chains U1 == ... == T == V1 == ..., then all the types that appear to the right of T must be types that the crate defining T already knows about. This rules out transitive chains where downstream crates can add new impls that โ€œstitch togetherโ€ comparisons of foreign types in ways that violate transitivity.

Not having such foreign impls also avoids forward compatibility issues where one crate adding more PartialEq implementations can cause build failures in downstream crates.

Derivable

This trait can be used with #[derive]. When derived on structs, two instances are equal if all fields are equal, and not equal if any fields are not equal. When derived on enums, two instances are equal if they are the same variant and all fields are equal.

How can I implement PartialEq?

An example implementation for a domain in which two books are considered the same book if their ISBN matches, even if the formats differ:

enum BookFormat {
    Paperback,
    Hardback,
    Ebook,
}

struct Book {
    isbn: i32,
    format: BookFormat,
}

impl PartialEq for Book {
    fn eq(&self, other: &Self) -> bool {
        self.isbn == other.isbn
    }
}

let b1 = Book { isbn: 3, format: BookFormat::Paperback };
let b2 = Book { isbn: 3, format: BookFormat::Ebook };
let b3 = Book { isbn: 10, format: BookFormat::Paperback };

assert!(b1 == b2);
assert!(b1 != b3);

How can I compare two different types?

The type you can compare with is controlled by PartialEqโ€™s type parameter. For example, letโ€™s tweak our previous code a bit:

// The derive implements <BookFormat> == <BookFormat> comparisons
#[derive(PartialEq)]
enum BookFormat {
    Paperback,
    Hardback,
    Ebook,
}

struct Book {
    isbn: i32,
    format: BookFormat,
}

// Implement <Book> == <BookFormat> comparisons
impl PartialEq<BookFormat> for Book {
    fn eq(&self, other: &BookFormat) -> bool {
        self.format == *other
    }
}

// Implement <BookFormat> == <Book> comparisons
impl PartialEq<Book> for BookFormat {
    fn eq(&self, other: &Book) -> bool {
        *self == other.format
    }
}

let b1 = Book { isbn: 3, format: BookFormat::Paperback };

assert!(b1 == BookFormat::Paperback);
assert!(BookFormat::Ebook != b1);

By changing impl PartialEq for Book to impl PartialEq<BookFormat> for Book, we allow BookFormats to be compared with Books.

A comparison like the one above, which ignores some fields of the struct, can be dangerous. It can easily lead to an unintended violation of the requirements for a partial equivalence relation. For example, if we kept the above implementation of PartialEq<Book> for BookFormat and added an implementation of PartialEq<Book> for Book (either via a #[derive] or via the manual implementation from the first example) then the result would violate transitivity:

โ“˜
#[derive(PartialEq)]
enum BookFormat {
    Paperback,
    Hardback,
    Ebook,
}

#[derive(PartialEq)]
struct Book {
    isbn: i32,
    format: BookFormat,
}

impl PartialEq<BookFormat> for Book {
    fn eq(&self, other: &BookFormat) -> bool {
        self.format == *other
    }
}

impl PartialEq<Book> for BookFormat {
    fn eq(&self, other: &Book) -> bool {
        *self == other.format
    }
}

fn main() {
    let b1 = Book { isbn: 1, format: BookFormat::Paperback };
    let b2 = Book { isbn: 2, format: BookFormat::Paperback };

    assert!(b1 == BookFormat::Paperback);
    assert!(BookFormat::Paperback == b2);

    // The following should hold by transitivity but doesn't.
    assert!(b1 == b2); // <-- PANICS
}

Examples

let x: u32 = 0;
let y: u32 = 1;

assert_eq!(x == y, false);
assert_eq!(x.eq(&y), false);

Required Methods

1.0.0 (const: unstable) ยท Source

fn eq(&self, other: &Rhs) -> bool

Tests for self and other values to be equal, and is used by ==.

Provided Methods

1.0.0 (const: unstable) ยท Source

fn ne(&self, other: &Rhs) -> bool

Tests for !=. The default implementation is almost always sufficient, and should not be overridden without very good reason.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl PartialEq for !

1.0.0 (const: unstable) ยท Source

impl PartialEq for ()

1.0.0 ยท Source

impl PartialEq for AddrParseError

Source

impl PartialEq for core::mem::Alignment

1.28.0 ยท Source

impl PartialEq for core::fmt::Alignment

Source

impl PartialEq for AllocError

Source

impl PartialEq for AsciiChar

Source

impl PartialEq for Assume

Source

impl PartialEq for AtomicOrdering

Source

impl PartialEq for ByteStr

1.64.0 ยท Source

impl PartialEq for CStr

Source

impl PartialEq for CharCase

1.34.0 ยท Source

impl PartialEq for CharTryFromError

1.27.0 ยท Source

impl PartialEq for CpuidResult

Available on x86 or x86-64 only.
Source

impl PartialEq for DebugAsHex

1.9.0 ยท Source

impl PartialEq for DecodeUtf16Error

1.3.0 ยท Source

impl PartialEq for Duration

1.0.0 ยท Source

impl PartialEq for Error

1.0.0 ยท Source

impl PartialEq for ErrorKind

Source

impl PartialEq for FieldId

Source

impl PartialEq for FormattingOptions

1.0.0 ยท Source

impl PartialEq for FpCategory

1.69.0 ยท Source

impl PartialEq for FromBytesUntilNulError

1.64.0 ยท Source

impl PartialEq for FromBytesWithNulError

1.86.0 ยท Source

impl PartialEq for GetDisjointMutError

1.34.0 (const: unstable) ยท Source

impl PartialEq for Infallible

1.55.0 ยท Source

impl PartialEq for IntErrorKind

1.7.0 ยท Source

impl PartialEq for IpAddr

1.0.0 (const: unstable) ยท Source

impl PartialEq for Ipv4Addr

1.0.0 (const: unstable) ยท Source

impl PartialEq for Ipv6Addr

Source

impl PartialEq for Ipv6MulticastScope

1.28.0 ยท Source

impl PartialEq for Layout

1.50.0 ยท Source

impl PartialEq for LayoutError

Source

impl PartialEq for Locality

1.10.0 ยท Source

impl PartialEq for Location<'_>

1.0.0 (const: unstable) ยท Source

impl PartialEq for core::cmp::Ordering

1.0.0 ยท Source

impl PartialEq for core::sync::atomic::Ordering

1.0.0 ยท Source

impl PartialEq for ParseBoolError

1.20.0 ยท Source

impl PartialEq for ParseCharError

1.0.0 ยท Source

impl PartialEq for ParseFloatError

1.0.0 ยท Source

impl PartialEq for ParseIntError

1.33.0 ยท Source

impl PartialEq for PhantomPinned

1.0.0 (const: unstable) ยท Source

impl PartialEq for RangeFull

1.36.0 ยท Source

impl PartialEq for RawWaker

1.36.0 ยท Source

impl PartialEq for RawWakerVTable

Source

impl PartialEq for SearchStep

Source

impl PartialEq for Sign

Source

impl PartialEq for SimdAlign

1.0.0 ยท Source

impl PartialEq for SocketAddr

1.0.0 ยท Source

impl PartialEq for SocketAddrV4

1.0.0 ยท Source

impl PartialEq for SocketAddrV6

1.59.0 ยท Source

impl PartialEq for TryFromCharError

1.66.0 ยท Source

impl PartialEq for TryFromFloatSecsError

1.34.0 ยท Source

impl PartialEq for TryFromIntError

1.0.0 (const: unstable) ยท Source

impl PartialEq for TypeId

1.0.0 ยท Source

impl PartialEq for Utf8Error

1.0.0 (const: unstable) ยท Source

impl PartialEq for bool

1.0.0 (const: unstable) ยท Source

impl PartialEq for char

1.0.0 (const: unstable) ยท Source

impl PartialEq for f16

1.0.0 (const: unstable) ยท Source

impl PartialEq for f32

1.0.0 (const: unstable) ยท Source

impl PartialEq for f64

1.0.0 (const: unstable) ยท Source

impl PartialEq for f128

1.0.0 (const: unstable) ยท Source

impl PartialEq for i8

1.0.0 (const: unstable) ยท Source

impl PartialEq for i16

1.0.0 (const: unstable) ยท Source

impl PartialEq for i32

1.0.0 (const: unstable) ยท Source

impl PartialEq for i64

1.0.0 (const: unstable) ยท Source

impl PartialEq for i128

1.0.0 (const: unstable) ยท Source

impl PartialEq for isize

1.0.0 (const: unstable) ยท Source

impl PartialEq for str

Source

impl PartialEq for svpattern

Available on AArch64 or ARM64EC only.
Source

impl PartialEq for svprfop

Available on AArch64 or ARM64EC only.
1.0.0 (const: unstable) ยท Source

impl PartialEq for u8

1.0.0 (const: unstable) ยท Source

impl PartialEq for u16

1.0.0 (const: unstable) ยท Source

impl PartialEq for u32

1.0.0 (const: unstable) ยท Source

impl PartialEq for u64

1.0.0 (const: unstable) ยท Source

impl PartialEq for u128

1.0.0 (const: unstable) ยท Source

impl PartialEq for usize

1.90.0 ยท Source

impl PartialEq<&CStr> for CStr

Source

impl PartialEq<&[u8]> for ByteStr

Source

impl PartialEq<&str> for ByteStr

Source

impl PartialEq<ByteStr> for &[u8]

Source

impl PartialEq<ByteStr> for &str

Source

impl PartialEq<ByteStr> for [u8]

Source

impl PartialEq<ByteStr> for str

1.16.0 ยท Source

impl PartialEq<IpAddr> for Ipv4Addr

1.16.0 ยท Source

impl PartialEq<IpAddr> for Ipv6Addr

1.16.0 ยท Source

impl PartialEq<Ipv4Addr> for IpAddr

1.16.0 ยท Source

impl PartialEq<Ipv6Addr> for IpAddr

Source

impl PartialEq<[u8]> for ByteStr

Source

impl PartialEq<str> for ByteStr

Source

impl<'a> PartialEq for PhantomContravariantLifetime<'a>

Source

impl<'a> PartialEq for PhantomCovariantLifetime<'a>

Source

impl<'a> PartialEq for PhantomInvariantLifetime<'a>

1.79.0 ยท Source

impl<'a> PartialEq for Utf8Chunk<'a>

Source

impl<'a> PartialEq for Utf8Pattern<'a>

1.0.0 (const: unstable) ยท Source

impl<A, B: PointeeSized> PartialEq<&B> for &A
where A: PartialEq<B> + PointeeSized,

1.0.0 (const: unstable) ยท Source

impl<A, B: PointeeSized> PartialEq<&B> for &mut A
where A: PartialEq<B> + PointeeSized,

1.0.0 (const: unstable) ยท Source

impl<A, B: PointeeSized> PartialEq<&mut B> for &A
where A: PartialEq<B> + PointeeSized,

1.0.0 (const: unstable) ยท Source

impl<A, B: PointeeSized> PartialEq<&mut B> for &mut A
where A: PartialEq<B> + PointeeSized,

1.55.0 (const: unstable) ยท Source

impl<B: PartialEq, C: PartialEq> PartialEq for ControlFlow<B, C>

Source

impl<Dyn: PointeeSized> PartialEq for DynMetadata<Dyn>

1.4.0 ยท Source

impl<F: FnPtr> PartialEq for F

1.29.0 ยท Source

impl<H> PartialEq for BuildHasherDefault<H>

1.0.0 (const: unstable) ยท Source

impl<Idx: PartialEq> PartialEq for core::ops::Range<Idx>

1.96.0 (const: unstable) ยท Source

impl<Idx: PartialEq> PartialEq for core::range::Range<Idx>

1.0.0 (const: unstable) ยท Source

impl<Idx: PartialEq> PartialEq for core::ops::RangeFrom<Idx>

1.96.0 (const: unstable) ยท Source

impl<Idx: PartialEq> PartialEq for core::range::RangeFrom<Idx>

1.26.0 (const: unstable) ยท Source

impl<Idx: PartialEq> PartialEq for core::ops::RangeInclusive<Idx>

1.95.0 ยท Source

impl<Idx: PartialEq> PartialEq for core::range::RangeInclusive<Idx>

1.0.0 (const: unstable) ยท Source

impl<Idx: PartialEq> PartialEq for RangeTo<Idx>

1.26.0 ยท Source

impl<Idx: PartialEq> PartialEq for core::ops::RangeToInclusive<Idx>

1.96.0 ยท Source

impl<Idx: PartialEq> PartialEq for core::range::RangeToInclusive<Idx>

1.41.0 ยท Source

impl<Ptr: Deref, Q: Deref> PartialEq<Pin<Q>> for Pin<Ptr>
where Ptr::Target: PartialEq<Q::Target>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<&[U]> for [T; N]
where T: PartialEq<U>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<&mut [U]> for [T; N]
where T: PartialEq<U>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<[U; N]> for &[T]
where T: PartialEq<U>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<[U; N]> for &mut [T]
where T: PartialEq<U>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<[U; N]> for [T; N]
where T: PartialEq<U>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<[U; N]> for [T]
where T: PartialEq<U>,

1.0.0 (const: unstable) ยท Source

impl<T, U, const N: usize> PartialEq<[U]> for [T; N]
where T: PartialEq<U>,

Source

impl<T, U> PartialEq<SyncView<U>> for SyncView<T>
where T: Sync + PartialEq<U> + ?Sized, U: Sync + ?Sized,

1.0.0 (const: unstable) ยท Source

impl<T, U> PartialEq<[U]> for [T]
where T: PartialEq<U>,

Source

impl<T, const N: usize> PartialEq for Mask<T, N>

Source

impl<T, const N: usize> PartialEq for Simd<T, N>

1.20.0 ยท Source

impl<T: ?Sized + PartialEq> PartialEq for ManuallyDrop<T>

1.0.0 ยท Source

impl<T: ?Sized + PartialEq> PartialEq for RefCell<T>

Source

impl<T: ?Sized, const VARIANT: u32, const FIELD: u32> PartialEq for FieldRepresentingType<T, VARIANT, FIELD>

1.0.0 ยท Source

impl<T: PartialEq + Copy> PartialEq for Cell<T>

Source

impl<T: PartialEq + PointeeSized> PartialEq for TraitImpl<T>

1.0.0 (const: unstable) ยท Source

impl<T: PartialEq, E: PartialEq> PartialEq for Result<T, E>

1.0.0 (const: unstable) ยท Source

impl<T: PartialEq> PartialEq for (Tโ‚, Tโ‚‚, โ€ฆ, Tโ‚™)

This trait is implemented for tuples up to twelve items long.

1.17.0 (const: unstable) ยท Source

impl<T: PartialEq> PartialEq for Bound<T>

1.0.0 ยท Source

impl<T: PartialEq> PartialEq for Cursor<T>

1.70.0 ยท Source

impl<T: PartialEq> PartialEq for OnceCell<T>

1.0.0 (const: unstable) ยท Source

impl<T: PartialEq> PartialEq for Option<T>

1.36.0 ยท Source

impl<T: PartialEq> PartialEq for Poll<T>

1.19.0 (const: unstable) ยท Source

impl<T: PartialEq> PartialEq for Reverse<T>

1.74.0 ยท Source

impl<T: PartialEq> PartialEq for Saturating<T>

1.0.0 ยท Source

impl<T: PartialEq> PartialEq for Wrapping<T>

1.0.0 ยท Source

impl<T: PointeeSized> PartialEq for *const T

Pointer equality is by address, as produced by the <*const T>::addr method.

1.0.0 ยท Source

impl<T: PointeeSized> PartialEq for *mut T

Pointer equality is by address, as produced by the <*mut T>::addr method.

1.25.0 ยท Source

impl<T: PointeeSized> PartialEq for NonNull<T>

1.0.0 ยท Source

impl<T: PointeeSized> PartialEq for PhantomData<T>

1.21.0 ยท Source

impl<T> PartialEq for Discriminant<T>

1.28.0 (const: unstable) ยท Source

impl<T> PartialEq for NonZero<T>

Source

impl<T> PartialEq for PhantomContravariant<T>
where T: ?Sized,

Source

impl<T> PartialEq for PhantomCovariant<T>
where T: ?Sized,

Source

impl<T> PartialEq for PhantomInvariant<T>
where T: ?Sized,

Source

impl<Y: PartialEq, R: PartialEq> PartialEq for CoroutineState<Y, R>

Source

impl<const N: usize> PartialEq<&[u8; N]> for ByteStr

Source

impl<const N: usize> PartialEq<ByteStr> for &[u8; N]

Source

impl<const N: usize> PartialEq<ByteStr> for [u8; N]

Source

impl<const N: usize> PartialEq<[u8; N]> for ByteStr